@@ -215,6 +215,37 @@ function dumpObjectTree (o, recurse, compress, level)
     
 }
 
+function getService(contractID, iface)
+{
+    var rv;
+    var cls = Components.classes[contractID];
+
+    if (!cls)
+        return null;
+
+    switch (typeof iface)
+    {
+        case "undefined":
+            rv = cls.getService();
+            break;
+
+        case "string":
+            rv = cls.getService(Components.interfaces[iface]);
+            break;
+
+        case "object":
+            rv = cls.getService(iface);
+            break;
+
+        default:
+            rv = null;
+            break;
+    }
+
+    return rv;
+
+}
+
 function safeHTML(str)
 {
     function replaceChars(ch)
@@ -229,12 +260,34 @@ function safeHTML(str)
                     
             case "&":
                 return "&amp;";
+                    
+            case "'":
+                return "&#39;";
+                    
+            case '"':
+                return "&quot;";
         }
 
         return "?";
     };
-        
-    return String(str).replace(/[<>&]/g, replaceChars);
+
+    return String(str).replace(/[<>&"']/g, replaceChars);
+}
+
+function safeCSV(str)
+{
+    function replaceChars(ch)
+    {
+        switch (ch)
+        {
+            case '"':
+                return '""';
+        }
+
+        return "?";
+    }
+
+    return '"' + String(str).replace(/"/g, replaceChars) + '"';
 }
 
 function alert(msg, parent, title)
@@ -270,14 +323,30 @@ function prompt(msg, initial, parent, title)
         parent = window;
     if (!title)
         title = MSG_PROMPT;
-    rv = { value: initial };
+    var rv = { value: initial };
 
     if (!ps.prompt (parent, title, msg, rv, null, {value: null}))
         return null;
 
-    return rv.value
+    return rv.value;
 }
 
+function alertCheck(msg, checkMsg, checkVal, parent, title)
+{
+    const PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    const nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].getService(nsIPromptService);
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_ALERT;
+    
+    var checkBoxWrapper = {value: !!checkVal};
+    ps.alertCheck(parent, title, msg, checkMsg, checkBoxWrapper);
+    return checkBoxWrapper.value;
+}
+
+
 function getChildById (element, id)
 {
     var nl = element.getElementsByAttribute("id", id);
@@ -617,7 +686,7 @@ function getSpecialDirectory(name)
 
 function getPathFromURL (url)
 {
-    var ary = url.match(/^(.*\/)([^\/?#]+)(\?|#|$)/);
+    var ary = url.match(/^(.*\/)([^\/?#]*)(\?|#|$)/);
     if (ary)
         return ary[1];
 
@@ -1022,3 +1091,15 @@ function makeExpression (items)
     
     return expression;
 }
+
+function isinstance(inst, base)
+{
+    /* Returns |true| if |inst| was constructed by |base|. Not 100% accurate,
+     * but plenty good enough for us. This is to work around the fix for bug
+     * 254067 which makes instanceof fail if the two sides are 'from'
+     * different windows (something we don't care about).
+     */
+    return (inst && base &&
+            ((inst instanceof base) ||
+             (inst.constructor && (inst.constructor.name == base.name))));
+}
