@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Peter Annema <disttsc@bart.nl>
  *   Dean Tessman <dean_tessman@hotmail.com>
+ *   Masayuki Nakano <masayuki@d-toybox.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -49,8 +50,7 @@
 #include "nsCOMPtr.h"
 #include "nsIDeviceContext.h"
 #include "nsIFontMetrics.h"
-#include "nsHTMLAtoms.h"
-#include "nsXULAtoms.h"
+#include "nsGkAtoms.h"
 #include "nsPresContext.h"
 #include "nsIRenderingContext.h"
 #include "nsStyleContext.h"
@@ -68,6 +68,8 @@
 #include "nsITheme.h"
 #include "nsUnicharUtils.h"
 #include "nsContentUtils.h"
+#include "nsDisplayList.h"
+#include "nsCSSRendering.h"
 #include "nsIReflowCallback.h"
 
 #ifdef IBMBIDI
@@ -75,8 +77,6 @@
 #include "nsBidiPresUtils.h"
 #endif // IBMBIDI
 
-#define ELLIPSIS "..."
-
 #define CROP_LEFT   "left"
 #define CROP_RIGHT  "right"
 #define CROP_CENTER "center"
@@ -101,61 +101,49 @@ PRBool nsTextBoxFrame::gInsertSeparatorPrefInitialized = PR_FALSE;
 //
 // NS_NewToolbarFrame
 //
-// Creates a new Toolbar frame and returns it in |aNewFrame|
+// Creates a new Toolbar frame and returns it
 //
-nsresult
-NS_NewTextBoxFrame ( nsIPresShell* aPresShell, nsIFrame** aNewFrame )
+nsIFrame*
+NS_NewTextBoxFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-    NS_PRECONDITION(aNewFrame, "null OUT ptr");
-    if (nsnull == aNewFrame) {
-        return NS_ERROR_NULL_POINTER;
-    }
-    nsTextBoxFrame* it = new (aPresShell) nsTextBoxFrame (aPresShell);
-    if (nsnull == it)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    // it->SetFlags(aFlags);
-    *aNewFrame = it;
-    return NS_OK;
-
+    return new (aPresShell) nsTextBoxFrame (aPresShell, aContext);
 } // NS_NewTextFrame
 
 
 NS_IMETHODIMP
-nsTextBoxFrame::AttributeChanged(nsIContent*     aChild,
-                                 PRInt32         aNameSpaceID,
+nsTextBoxFrame::AttributeChanged(PRInt32         aNameSpaceID,
                                  nsIAtom*        aAttribute,
                                  PRInt32         aModType)
 {
     mState |= NS_STATE_NEED_LAYOUT;
     PRBool aResize;
     PRBool aRedraw;
-    nsPresContext* presContext = GetPresContext();
 
-    UpdateAttributes(presContext, aAttribute, aResize, aRedraw);
+    UpdateAttributes(aAttribute, aResize, aRedraw);
 
     if (aResize) {
-        nsBoxLayoutState state(presContext);
-        MarkDirty(state);
+        PresContext()->PresShell()->
+            FrameNeedsReflow(this, nsIPresShell::eStyleChange,
+                             NS_FRAME_IS_DIRTY);
     } else if (aRedraw) {
-        nsBoxLayoutState state(presContext);
+        nsBoxLayoutState state(PresContext());
         Redraw(state);
     }
 
     // If the accesskey changed, register for the new value
     // The old value has been unregistered in nsXULElement::SetAttr
-    if (aAttribute == nsXULAtoms::accesskey || aAttribute == nsXULAtoms::control)
-        RegUnregAccessKey(presContext, PR_TRUE);
+    if (aAttribute == nsGkAtoms::accesskey || aAttribute == nsGkAtoms::control)
+        RegUnregAccessKey(PR_TRUE);
 
     return NS_OK;
 }
 
-nsTextBoxFrame::nsTextBoxFrame(nsIPresShell* aShell)
-  : nsLeafBoxFrame(aShell), mCropType(CropRight), mAccessKeyInfo(nsnull), 
-    mNeedsReflowCallback(PR_FALSE)
+nsTextBoxFrame::nsTextBoxFrame(nsIPresShell* aShell, nsStyleContext* aContext):
+  nsLeafBoxFrame(aShell, aContext), mCropType(CropRight), mAccessKeyInfo(nsnull),
+  mNeedsReflowCallback(PR_FALSE)
 {
     mState |= NS_STATE_NEED_LAYOUT;
-    NeedsRecalc();
+    MarkIntrinsicWidthsDirty();
 }
 
 nsTextBoxFrame::~nsTextBoxFrame()
@@ -165,34 +153,29 @@ nsTextBoxFrame::~nsTextBoxFrame()
 
 
 NS_IMETHODIMP
-nsTextBoxFrame::Init(nsPresContext*  aPresContext,
-                     nsIContent*      aContent,
+nsTextBoxFrame::Init(nsIContent*      aContent,
                      nsIFrame*        aParent,
-                     nsStyleContext*  aContext,
                      nsIFrame*        aPrevInFlow)
 {
-    nsresult rv = nsTextBoxFrameSuper::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
-    if (NS_FAILED(rv))
-        return rv;
+    nsTextBoxFrameSuper::Init(aContent, aParent, aPrevInFlow);
 
     mState |= NS_STATE_NEED_LAYOUT;
     PRBool aResize;
     PRBool aRedraw;
-    UpdateAttributes(aPresContext, nsnull, aResize, aRedraw); /* update all */
+    UpdateAttributes(nsnull, aResize, aRedraw); /* update all */
 
     // register access key
-    RegUnregAccessKey(aPresContext, PR_TRUE);
+    RegUnregAccessKey(PR_TRUE);
 
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::Destroy(nsPresContext* aPresContext)
+void
+nsTextBoxFrame::Destroy()
 {
     // unregister access key
-    RegUnregAccessKey(aPresContext, PR_FALSE);
-
-    return nsTextBoxFrameSuper::Destroy(aPresContext);
+    RegUnregAccessKey(PR_FALSE);
+    nsTextBoxFrameSuper::Destroy();
 }
 
 PRBool
@@ -230,23 +213,26 @@ public:
     {
     }
 
-    NS_DECL_ISUPPORTS
-
-    NS_IMETHOD ReflowFinished(nsIPresShell* aShell, PRBool* aFlushFlag)
+    virtual PRBool ReflowFinished()
     {
+        PRBool shouldFlush = PR_FALSE;
         nsTextBoxFrame* frame =
-            NS_STATIC_CAST(nsTextBoxFrame*, mWeakFrame.GetFrame());
-        if (frame && frame->UpdateAccesskey(mWeakFrame)) {
-            *aFlushFlag = PR_TRUE;
+            static_cast<nsTextBoxFrame*>(mWeakFrame.GetFrame());
+        if (frame) {
+            shouldFlush = frame->UpdateAccesskey(mWeakFrame);
         }
-        return NS_OK;
+        delete this;
+        return shouldFlush;
+    }
+
+    virtual void ReflowCallbackCanceled()
+    {
+        delete this;
     }
 
     nsWeakFrame mWeakFrame;
 };
 
-NS_IMPL_ISUPPORTS1(nsAsyncAccesskeyUpdate, nsIReflowCallback)
-
 PRBool
 nsTextBoxFrame::UpdateAccesskey(nsWeakFrame& aWeakThis)
 {
@@ -254,30 +240,35 @@ nsTextBoxFrame::UpdateAccesskey(nsWeakFrame& aWeakThis)
     nsCOMPtr<nsIDOMXULLabelElement> labelElement = do_QueryInterface(mContent);
     if (labelElement) {
         // Accesskey may be stored on control.
-        nsCxPusher cx(mContent);
-        labelElement->GetAccessKey(accesskey);
-        NS_ENSURE_TRUE(aWeakThis.IsAlive(), PR_FALSE);
+        // Because this method is called by the reflow callback, current context
+        // may not be the right one. Pushing the context of mContent so that
+        // if nsIDOMXULLabelElement is implemented in XBL, we don't get a
+        // security exception.
+        nsCxPusher cx;
+        if (cx.Push(mContent)) {
+          labelElement->GetAccessKey(accesskey);
+          NS_ENSURE_TRUE(aWeakThis.IsAlive(), PR_FALSE);
+        }
     }
     else {
-        mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, accesskey);
+        mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::accesskey, accesskey);
     }
 
     if (!accesskey.Equals(mAccessKey)) {
         // Need to get clean mTitle.
-        mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, mTitle);
+        mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::value, mTitle);
         mAccessKey = accesskey;
         UpdateAccessTitle();
-        nsPresContext* presContext = GetPresContext();
-        nsBoxLayoutState state(presContext);
-        MarkDirty(state);
+        PresContext()->PresShell()->
+            FrameNeedsReflow(this, nsIPresShell::eStyleChange,
+                             NS_FRAME_IS_DIRTY);
         return PR_TRUE;
     }
     return PR_FALSE;
 }
 
 void
-nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
-                                 nsIAtom*         aAttribute,
+nsTextBoxFrame::UpdateAttributes(nsIAtom*         aAttribute,
                                  PRBool&          aResize,
                                  PRBool&          aRedraw)
 {
@@ -285,21 +276,28 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
     aResize = PR_FALSE;
     aRedraw = PR_FALSE;
 
-    if (aAttribute == nsnull || aAttribute == nsXULAtoms::crop) {
-        nsAutoString value;
-        mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::crop, value);
+    if (aAttribute == nsnull || aAttribute == nsGkAtoms::crop) {
+        static nsIContent::AttrValuesArray strings[] =
+          {&nsGkAtoms::left, &nsGkAtoms::start, &nsGkAtoms::center,
+           &nsGkAtoms::right, &nsGkAtoms::end, nsnull};
         CroppingStyle cropType;
-
-        if (value.EqualsASCII(CROP_LEFT) ||
-            value.EqualsASCII(CROP_START))
+        switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::crop,
+                                          strings, eCaseMatters)) {
+          case 0:
+          case 1:
             cropType = CropLeft;
-        else if (value.EqualsASCII(CROP_CENTER))
+            break;
+          case 2:
             cropType = CropCenter;
-        else if (value.EqualsASCII(CROP_RIGHT) ||
-                 value.EqualsASCII(CROP_END))
+            break;
+          case 3:
+          case 4:
             cropType = CropRight;
-        else
+            break;
+          default:
             cropType = CropNone;
+            break;
+        }
 
         if (cropType != mCropType) {
             aResize = PR_TRUE;
@@ -307,12 +305,12 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
         }
     }
 
-    if (aAttribute == nsnull || aAttribute == nsHTMLAtoms::value) {
-        mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, mTitle);
+    if (aAttribute == nsnull || aAttribute == nsGkAtoms::value) {
+        mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::value, mTitle);
         doUpdateTitle = PR_TRUE;
     }
 
-    if (aAttribute == nsnull || aAttribute == nsXULAtoms::accesskey) {
+    if (aAttribute == nsnull || aAttribute == nsGkAtoms::accesskey) {
         mNeedsReflowCallback = PR_TRUE;
         // Ensure that layout is refreshed and reflow callback called.
         aResize = PR_TRUE;
@@ -325,47 +323,61 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
 
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::Paint(nsPresContext*      aPresContext,
-                      nsIRenderingContext& aRenderingContext,
-                      const nsRect&        aDirtyRect,
-                      nsFramePaintLayer    aWhichLayer,
-                      PRUint32             aFlags)
+class nsDisplayXULTextBox : public nsDisplayItem {
+public:
+  nsDisplayXULTextBox(nsTextBoxFrame* aFrame) : nsDisplayItem(aFrame) {
+      MOZ_COUNT_CTOR(nsDisplayXULTextBox);
+  }
+#ifdef NS_BUILD_REFCNT_LOGGING
+  virtual ~nsDisplayXULTextBox() {
+      MOZ_COUNT_DTOR(nsDisplayXULTextBox);
+  }
+#endif
+
+  virtual void Paint(nsDisplayListBuilder* aBuilder, nsIRenderingContext* aCtx,
+     const nsRect& aDirtyRect);
+  NS_DISPLAY_DECL_NAME("XULTextBox")
+};
+
+void nsDisplayXULTextBox::Paint(nsDisplayListBuilder* aBuilder,
+     nsIRenderingContext* aCtx, const nsRect& aDirtyRect)
 {
-    if (!GetStyleVisibility()->IsVisible())
-        return NS_OK;
-
-    if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
-
-        // remove the border and padding
-        nsStyleBorderPadding  bPad;
-        mStyleContext->GetBorderPaddingFor(bPad);
-        nsMargin border(0,0,0,0);
-        bPad.GetBorderPadding(border);
-
-        nsRect textRect(0,0,mRect.width, mRect.height);
-        textRect.Deflate(border);
-
-        PaintTitle(aPresContext, aRenderingContext, aDirtyRect, textRect);
-    }
-
-    return nsLeafFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  static_cast<nsTextBoxFrame*>(mFrame)->
+    PaintTitle(*aCtx, aDirtyRect, aBuilder->ToReferenceFrame(mFrame));
 }
 
 NS_IMETHODIMP
-nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
-                           nsIRenderingContext& aRenderingContext,
+nsTextBoxFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                 const nsRect&           aDirtyRect,
+                                 const nsDisplayListSet& aLists)
+{
+    if (!IsVisibleForPainting(aBuilder))
+      return NS_OK;
+
+    nsresult rv = nsLeafBoxFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    return aLists.Content()->AppendNewToTop(new (aBuilder)
+        nsDisplayXULTextBox(this));
+}
+
+void
+nsTextBoxFrame::PaintTitle(nsIRenderingContext& aRenderingContext,
                            const nsRect&        aDirtyRect,
-                           const nsRect&        aRect)
+                           nsPoint              aPt)
 {
     if (mTitle.IsEmpty())
-        return NS_OK;
+        return;
+
+    nsRect textRect(aPt, GetSize());
+    textRect.Deflate(GetUsedBorderAndPadding());
 
     // determine (cropped) title and underline position
-    LayoutTitle(aPresContext, aRenderingContext, aRect);
+    nsPresContext* presContext = PresContext();
+    LayoutTitle(presContext, aRenderingContext, textRect);
 
     // make the rect as small as our (cropped) text.
-    nsRect textRect(aRect);
+    nscoord outerWidth = textRect.width;
     textRect.width = mTitleWidth;
 
     // Align our text within the overall rect by checking our text-align property.
@@ -373,19 +385,19 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
     const nsStyleText* textStyle = GetStyleText();
 
     if (textStyle->mTextAlign == NS_STYLE_TEXT_ALIGN_CENTER)
-      textRect.x += (aRect.width - textRect.width)/2;
+      textRect.x += (outerWidth - textRect.width)/2;
     else if (textStyle->mTextAlign == NS_STYLE_TEXT_ALIGN_RIGHT) {
       if (vis->mDirection == NS_STYLE_DIRECTION_LTR)
-        textRect.x += (aRect.width - textRect.width);
+        textRect.x += (outerWidth - textRect.width);
     }
     else {
       if (vis->mDirection == NS_STYLE_DIRECTION_RTL)
-        textRect.x += (aRect.width - textRect.width);
+        textRect.x += (outerWidth - textRect.width);
     }
 
     // don't draw if the title is not dirty
     if (PR_FALSE == aDirtyRect.Intersects(textRect))
-        return NS_OK;
+        return;
 
     // paint the title
     nscolor overColor;
@@ -428,34 +440,52 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
       }
     } while (context && hasDecorations && (0 != decorMask));
 
-    const nsStyleFont* fontStyle = GetStyleFont();
-    
+    nsCOMPtr<nsIFontMetrics> fontMet;
+    nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fontMet));
+
     nscoord offset;
     nscoord size;
-    nscoord baseline;
-    nsCOMPtr<nsIFontMetrics> fontMet;
-    aPresContext->DeviceContext()->GetMetricsFor(fontStyle->mFont,
-                                                 *getter_AddRefs(fontMet));
-    fontMet->GetMaxAscent(baseline);
+    nscoord ascent;
+    fontMet->GetMaxAscent(ascent);
 
+    nscoord baseline =
+      presContext->RoundAppUnitsToNearestDevPixels(textRect.y + ascent);
+    nsRefPtr<gfxContext> ctx = aRenderingContext.ThebesContext();
+    gfxPoint pt(presContext->AppUnitsToGfxUnits(textRect.x),
+                presContext->AppUnitsToGfxUnits(textRect.y));
+    gfxFloat width = presContext->AppUnitsToGfxUnits(textRect.width);
+    gfxFloat ascentPixel = presContext->AppUnitsToGfxUnits(ascent);
     if (decorations & (NS_FONT_DECORATION_OVERLINE | NS_FONT_DECORATION_UNDERLINE)) {
       fontMet->GetUnderline(offset, size);
+      gfxFloat offsetPixel = presContext->AppUnitsToGfxUnits(offset);
+      gfxFloat sizePixel = presContext->AppUnitsToGfxUnits(size);
       if (decorations & NS_FONT_DECORATION_OVERLINE) {
-        aRenderingContext.SetColor(overColor);
-        aRenderingContext.FillRect(textRect.x, textRect.y, textRect.width, size);
+        nsCSSRendering::PaintDecorationLine(ctx, overColor,
+                                            pt, gfxSize(width, sizePixel),
+                                            ascentPixel, ascentPixel,
+                                            NS_STYLE_TEXT_DECORATION_OVERLINE,
+                                            NS_STYLE_BORDER_STYLE_SOLID);
       }
       if (decorations & NS_FONT_DECORATION_UNDERLINE) {
-        aRenderingContext.SetColor(underColor);
-        aRenderingContext.FillRect(textRect.x, textRect.y + baseline - offset, textRect.width, size);
+        nsCSSRendering::PaintDecorationLine(ctx, underColor,
+                                            pt, gfxSize(width, sizePixel),
+                                            ascentPixel, offsetPixel,
+                                            NS_STYLE_TEXT_DECORATION_UNDERLINE,
+                                            NS_STYLE_BORDER_STYLE_SOLID);
       }
     }
     if (decorations & NS_FONT_DECORATION_LINE_THROUGH) {
       fontMet->GetStrikeout(offset, size);
-      aRenderingContext.SetColor(strikeColor);
-      aRenderingContext.FillRect(textRect.x, textRect.y + baseline - offset, textRect.width, size);
+      gfxFloat offsetPixel = presContext->AppUnitsToGfxUnits(offset);
+      gfxFloat sizePixel = presContext->AppUnitsToGfxUnits(size);
+      nsCSSRendering::PaintDecorationLine(ctx, underColor,
+                                          pt, gfxSize(width, sizePixel),
+                                          ascentPixel, offsetPixel,
+                                          NS_STYLE_TEXT_DECORATION_LINE_THROUGH,
+                                          NS_STYLE_BORDER_STYLE_SOLID);
     }
- 
-    aRenderingContext.SetFont(fontStyle->mFont, nsnull);
+
+    aRenderingContext.SetFont(fontMet);
 
     CalculateUnderline(aRenderingContext);
 
@@ -465,8 +495,8 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
     nsresult rv = NS_ERROR_FAILURE;
 
     if (mState & NS_FRAME_IS_BIDI) {
-      aPresContext->SetBidiEnabled(PR_TRUE);
-      nsBidiPresUtils* bidiUtils = aPresContext->GetBidiUtils();
+      presContext->SetBidiEnabled(PR_TRUE);
+      nsBidiPresUtils* bidiUtils = presContext->GetBidiUtils();
 
       if (bidiUtils) {
         const nsStyleVisibility* vis = GetStyleVisibility();
@@ -477,8 +507,8 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
            nsBidiPositionResolve posResolve;
            posResolve.logicalIndex = mAccessKeyInfo->mAccesskeyIndex;
            rv = bidiUtils->RenderText(mCroppedTitle.get(), mCroppedTitle.Length(), direction,
-                                      aPresContext, aRenderingContext,
-                                      textRect.x, textRect.y + baseline,
+                                      presContext, aRenderingContext,
+                                      textRect.x, baseline,
                                       &posResolve,
                                       1);
            mAccessKeyInfo->mBeforeWidth = posResolve.visualLeftTwips;
@@ -486,14 +516,16 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
         else
         {
            rv = bidiUtils->RenderText(mCroppedTitle.get(), mCroppedTitle.Length(), direction,
-                                      aPresContext, aRenderingContext,
-                                      textRect.x, textRect.y + baseline);
+                                      presContext, aRenderingContext,
+                                      textRect.x, baseline);
         }
       }
     }
     if (NS_FAILED(rv) )
 #endif // IBMBIDI
     {
+       aRenderingContext.SetTextRunRTL(PR_FALSE);
+
        if (mAccessKeyInfo && mAccessKeyInfo->mAccesskeyIndex != kNotFound) {
            // In the simple (non-BiDi) case, we calculate the mnemonic's
            // underline position by getting the text metric.
@@ -505,7 +537,7 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
                mAccessKeyInfo->mBeforeWidth = 0;
        }
 
-       aRenderingContext.DrawString(mCroppedTitle, textRect.x, textRect.y + baseline);
+       aRenderingContext.DrawString(mCroppedTitle, textRect.x, baseline);
     }
 
     if (mAccessKeyInfo && mAccessKeyInfo->mAccesskeyIndex != kNotFound) {
@@ -514,8 +546,6 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
                                    mAccessKeyInfo->mAccessWidth,
                                    mAccessKeyInfo->mAccessUnderlineSize);
     }
-
-    return NS_OK;
 }
 
 void
@@ -542,8 +572,9 @@ nsTextBoxFrame::CalculateUnderline(nsIRenderingContext& aRenderingContext)
 {
     if (mAccessKeyInfo && mAccessKeyInfo->mAccesskeyIndex != kNotFound) {
          // Calculate all fields of mAccessKeyInfo which
-         // are the same for both BiDi and non-BiDi rames.
+         // are the same for both BiDi and non-BiDi frames.
          const PRUnichar *titleString = mCroppedTitle.get();
+         aRenderingContext.SetTextRunRTL(PR_FALSE);
          aRenderingContext.GetWidth(titleString[mAccessKeyInfo->mAccesskeyIndex],
                                     mAccessKeyInfo->mAccessWidth);
 
@@ -565,40 +596,31 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
     if (mTitle.IsEmpty())
         return;
 
-    nsCOMPtr<nsIFontMetrics> fontMet;
-    aPresContext->DeviceContext()->GetMetricsFor(GetStyleFont()->mFont,
-                                                 *getter_AddRefs(fontMet));
-    aRenderingContext.SetFont(fontMet);
+    nsLayoutUtils::SetFontFromStyle(&aRenderingContext, GetStyleContext());
 
     // see if the text will completely fit in the width given
-    aRenderingContext.GetWidth(mTitle, mTitleWidth);
+    mTitleWidth = nsLayoutUtils::GetStringWidth(this, &aRenderingContext,
+                                                mTitle.get(), mTitle.Length());
 
     if (mTitleWidth <= aWidth) {
         mCroppedTitle = mTitle;
 #ifdef IBMBIDI
-        PRInt32 length = mTitle.Length();
-        for (PRInt32 i = 0; i < length; i++) {
-          if ((UCS2_CHAR_IS_BIDI(mTitle.CharAt(i)) ) ||
-              ((IS_HIGH_SURROGATE(mTitle.CharAt(i))) &&
-               (++i < length) &&
-               (IS_LOW_SURROGATE(mTitle.CharAt(i))) &&
-               (UTF32_CHAR_IS_BIDI(SURROGATE_TO_UCS4(mTitle.CharAt(i-1),
-                                                     mTitle.CharAt(i)))))) {
+        if (HasRTLChars(mTitle)) {
             mState |= NS_FRAME_IS_BIDI;
-            break;
-          }
         }
 #endif // IBMBIDI
         return;  // fits, done.
     }
 
+    const nsDependentString& kEllipsis = nsContentUtils::GetLocalizedEllipsis();
     // start with an ellipsis
-    mCroppedTitle.AssignASCII(ELLIPSIS);
+    mCroppedTitle.Assign(kEllipsis);
 
     // see if the width is even smaller than the ellipsis
     // if so, clear the text (XXX set as many '.' as we can?).
     nscoord ellipsisWidth;
-    aRenderingContext.GetWidth(ELLIPSIS, ellipsisWidth);
+    aRenderingContext.SetTextRunRTL(PR_FALSE);
+    aRenderingContext.GetWidth(kEllipsis, ellipsisWidth);
 
     if (ellipsisWidth > aWidth) {
         mCroppedTitle.SetLength(0);
@@ -615,6 +637,7 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
     aWidth -= ellipsisWidth;
 
     // XXX: This whole block should probably take surrogates into account
+    // XXX and clusters!
     // ok crop things
     switch (mCropType)
     {
@@ -627,6 +650,7 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
             int i;
             for (i = 0; i < length; ++i) {
                 PRUnichar ch = mTitle.CharAt(i);
+                // still in LTR mode
                 aRenderingContext.GetWidth(ch,cwidth);
                 if (twidth + cwidth > aWidth)
                     break;
@@ -680,8 +704,9 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
 
         case CropCenter:
         {
-            nscoord stringWidth = 0;
-            aRenderingContext.GetWidth(mTitle, stringWidth);
+            nscoord stringWidth =
+                nsLayoutUtils::GetStringWidth(this, &aRenderingContext,
+                                              mTitle.get(), mTitle.Length());
             if (stringWidth <= aWidth) {
                 // the entire string will fit in the maximum width
                 mCroppedTitle.Insert(mTitle, 0);
@@ -696,6 +721,7 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
             nsAutoString leftString, rightString;
 
             rightPos = mTitle.Length() - 1;
+            aRenderingContext.SetTextRunRTL(PR_FALSE);
             for (leftPos = 0; leftPos <= rightPos;) {
                 // look at the next character on the left end
                 ch = mTitle.CharAt(leftPos);
@@ -733,53 +759,72 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
                 rightPos--;
             }
 
-            // form the new cropped string
-            nsAutoString ellipsisString;
-            ellipsisString.AssignASCII(ELLIPSIS);
-
-            mCroppedTitle = leftString + ellipsisString + rightString;
+            mCroppedTitle = leftString + kEllipsis + rightString;
         }
         break;
     }
 
-    aRenderingContext.GetWidth(mCroppedTitle, mTitleWidth);
+    mTitleWidth = nsLayoutUtils::GetStringWidth(this, &aRenderingContext,
+                                                mCroppedTitle.get(), mCroppedTitle.Length());
 }
 
+#define OLD_ELLIPSIS NS_LITERAL_STRING("...")
+
 // the following block is to append the accesskey to mTitle if there is an accesskey
 // but the mTitle doesn't have the character
 void
 nsTextBoxFrame::UpdateAccessTitle()
 {
+    /*
+     * Note that if you change appending access key label spec,
+     * you need to maintain same logic in following methods. See bug 324159.
+     * toolkit/content/commonDialog.js (setLabelForNode)
+     * toolkit/content/widgets/text.xml (formatAccessKey)
+     * xpfe/global/resources/content/commonDialog.js (setLabelForNode)
+     * xpfe/global/resources/content/bindings/text.xml (formatAccessKey)
+     */
     PRInt32 menuAccessKey;
     nsMenuBarListener::GetMenuAccessKey(&menuAccessKey);
-    if (menuAccessKey) {
-        if (!mAccessKey.IsEmpty()) {
-            if (( !FindInReadable(mAccessKey, mTitle, nsCaseInsensitiveStringComparator()))
-                || AlwaysAppendAccessKey()) 
-            {
-                nsAutoString tmpstring;
-                if (InsertSeparatorBeforeAccessKey() &&
-                    !mTitle.IsEmpty() && !NS_IS_SPACE(mTitle.Last())) {
-                  tmpstring += ' ';
-                }
-                tmpstring += '(';
-                tmpstring += mAccessKey;
-                ToUpperCase(tmpstring);
-                tmpstring.Append(NS_LITERAL_STRING(")"));
-                PRInt32 offset = mTitle.RFind("...");
-                if (offset != kNotFound) {
-                    mTitle.Insert(tmpstring,NS_STATIC_CAST(PRUint32, offset));
-                } else {
-                    PRUint32 l = mTitle.Length();
-                    if((l > 0) && (PRUnichar(':')==mTitle[l-1])) {
-                      mTitle.Insert(tmpstring,l-1);
-                    } else {
-                      mTitle += tmpstring;
-                    }
-                }
-            }
-        }
+    if (!menuAccessKey || mAccessKey.IsEmpty())
+        return;
+
+    if (!AlwaysAppendAccessKey() &&
+        FindInReadable(mAccessKey, mTitle, nsCaseInsensitiveStringComparator()))
+        return;
+
+    nsAutoString accessKeyLabel;
+    accessKeyLabel += '(';
+    accessKeyLabel += mAccessKey;
+    ToUpperCase(accessKeyLabel);
+    accessKeyLabel += ')';
+
+    if (mTitle.IsEmpty()) {
+        mTitle = accessKeyLabel;
+        return;
     }
+
+    const nsDependentString& kEllipsis = nsContentUtils::GetLocalizedEllipsis();
+    PRUint32 offset = mTitle.Length();
+    if (StringEndsWith(mTitle, kEllipsis)) {
+        offset -= kEllipsis.Length();
+    } else if (StringEndsWith(mTitle, OLD_ELLIPSIS)) {
+        // Try to check with our old ellipsis (for old addons)
+        offset -= OLD_ELLIPSIS.Length();
+    } else {
+        // Try to check with
+        // our default ellipsis (for non-localized addons) or ':'
+        const PRUnichar kLastChar = mTitle.Last();
+        if (kLastChar == PRUnichar(0x2026) || kLastChar == PRUnichar(':'))
+            offset--;
+    }
+
+    if (InsertSeparatorBeforeAccessKey() &&
+        !NS_IS_SPACE(mTitle[offset - 1])) {
+        mTitle.Insert(' ', offset);
+        offset++;
+    }
+
+    mTitle.Insert(accessKeyLabel, offset);
 }
 
 void
@@ -836,9 +881,9 @@ NS_IMETHODIMP
 nsTextBoxFrame::DoLayout(nsBoxLayoutState& aBoxLayoutState)
 {
     if (mNeedsReflowCallback) {
-        nsCOMPtr<nsIReflowCallback> cb = new nsAsyncAccesskeyUpdate(this);
+        nsIReflowCallback* cb = new nsAsyncAccesskeyUpdate(this);
         if (cb) {
-            GetPresContext()->PresShell()->PostReflowCallback(cb);
+            PresContext()->PresShell()->PostReflowCallback(cb);
         }
         mNeedsReflowCallback = PR_FALSE;
     }
@@ -848,11 +893,11 @@ nsTextBoxFrame::DoLayout(nsBoxLayoutState& aBoxLayoutState)
     return nsLeafBoxFrame::DoLayout(aBoxLayoutState);
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::NeedsRecalc()
+/* virtual */ void
+nsTextBoxFrame::MarkIntrinsicWidthsDirty()
 {
     mNeedsRecalc = PR_TRUE;
-    return NS_OK;
+    nsTextBoxFrameSuper::MarkIntrinsicWidthsDirty();
 }
 
 void
@@ -860,11 +905,11 @@ nsTextBoxFrame::GetTextSize(nsPresContext* aPresContext, nsIRenderingContext& aR
                                 const nsString& aString, nsSize& aSize, nscoord& aAscent)
 {
     nsCOMPtr<nsIFontMetrics> fontMet;
-    aPresContext->DeviceContext()->GetMetricsFor(GetStyleFont()->mFont,
-                                                 *getter_AddRefs(fontMet));
+    nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fontMet));
     fontMet->GetHeight(aSize.height);
     aRenderingContext.SetFont(fontMet);
-    aRenderingContext.GetWidth(aString, aSize.width);
+    aSize.width =
+      nsLayoutUtils::GetStringWidth(this, &aRenderingContext, aString.get(), aString.Length());
     fontMet->GetMaxAscent(aAscent);
 }
 
@@ -875,12 +920,7 @@ nsTextBoxFrame::CalcTextSize(nsBoxLayoutState& aBoxLayoutState)
     {
         nsSize size;
         nsPresContext* presContext = aBoxLayoutState.PresContext();
-        const nsHTMLReflowState* rstate = aBoxLayoutState.GetReflowState();
-        if (!rstate)
-            return;
-
-        nsIRenderingContext* rendContext = rstate->rendContext;
-
+        nsIRenderingContext* rendContext = aBoxLayoutState.GetRenderingContext();
         if (rendContext) {
             GetTextSize(presContext, *rendContext,
                         mTitle, size, mAscent);
@@ -893,63 +933,60 @@ nsTextBoxFrame::CalcTextSize(nsBoxLayoutState& aBoxLayoutState)
 /**
  * Ok return our dimensions
  */
-NS_IMETHODIMP
-nsTextBoxFrame::GetPrefSize(nsBoxLayoutState& aBoxLayoutState, nsSize& aSize)
+nsSize
+nsTextBoxFrame::GetPrefSize(nsBoxLayoutState& aBoxLayoutState)
 {
     CalcTextSize(aBoxLayoutState);
 
-    aSize = mTextSize;
+    nsSize size = mTextSize;
+    DISPLAY_PREF_SIZE(this, size);
 
-    AddBorderAndPadding(aSize);
-    AddInset(aSize);
-    nsIBox::AddCSSPrefSize(aBoxLayoutState, this, aSize);
+    AddBorderAndPadding(size);
+    nsIBox::AddCSSPrefSize(aBoxLayoutState, this, size);
 
-    return NS_OK;
+    return size;
 }
 
 /**
  * Ok return our dimensions
  */
-NS_IMETHODIMP
-nsTextBoxFrame::GetMinSize(nsBoxLayoutState& aBoxLayoutState, nsSize& aSize)
+nsSize
+nsTextBoxFrame::GetMinSize(nsBoxLayoutState& aBoxLayoutState)
 {
     CalcTextSize(aBoxLayoutState);
 
-    aSize = mTextSize;
+    nsSize size = mTextSize;
+    DISPLAY_MIN_SIZE(this, size);
 
-     // if there is cropping our min width becomes our border and  padding
-    if (mCropType != CropNone) {
-       aSize.width = 0;
-    }
+    // if there is cropping our min width becomes our border and padding
+    if (mCropType != CropNone)
+        size.width = 0;
 
-    AddBorderAndPadding(aSize);
-    AddInset(aSize);
-    nsIBox::AddCSSMinSize(aBoxLayoutState, this, aSize);
+    AddBorderAndPadding(size);
+    nsIBox::AddCSSMinSize(aBoxLayoutState, this, size);
 
-    return NS_OK;
+    return size;
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::GetAscent(nsBoxLayoutState& aBoxLayoutState, nscoord& aAscent)
+nscoord
+nsTextBoxFrame::GetBoxAscent(nsBoxLayoutState& aBoxLayoutState)
 {
     CalcTextSize(aBoxLayoutState);
 
-    aAscent = mAscent;
+    nscoord ascent = mAscent;
 
     nsMargin m(0,0,0,0);
     GetBorderAndPadding(m);
-    aAscent += m.top;
-    GetInset(m);
-    aAscent += m.top;
+    ascent += m.top;
 
-    return NS_OK;
+    return ascent;
 }
 
 #ifdef DEBUG
 NS_IMETHODIMP
 nsTextBoxFrame::GetFrameName(nsAString& aResult) const
 {
-    MakeFrameName(NS_LITERAL_STRING("Text"), aResult);
+    MakeFrameName(NS_LITERAL_STRING("TextBox"), aResult);
     aResult += NS_LITERAL_STRING("[value=") + mTitle + NS_LITERAL_STRING("]");
     return NS_OK;
 }
@@ -958,8 +995,7 @@ nsTextBoxFrame::GetFrameName(nsAString& aResult) const
 // If you make changes to this function, check its counterparts 
 // in nsBoxFrame and nsAreaFrame
 nsresult
-nsTextBoxFrame::RegUnregAccessKey(nsPresContext* aPresContext,
-                                  PRBool          aDoReg)
+nsTextBoxFrame::RegUnregAccessKey(PRBool aDoReg)
 {
     // if we have no content, we can't do anything
     if (!mContent)
@@ -973,12 +1009,12 @@ nsTextBoxFrame::RegUnregAccessKey(nsPresContext* aPresContext,
     // in e.g. <menu>, <menuitem>, <button>. These <label>s inherit
     // |accesskey| and would otherwise register themselves, overwriting
     // the content we really meant to be registered.
-    if (!mContent->HasAttr(kNameSpaceID_None, nsXULAtoms::control))
+    if (!mContent->HasAttr(kNameSpaceID_None, nsGkAtoms::control))
         return NS_OK;
 
     // see if we even have an access key
     nsAutoString accessKey;
-    mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, accessKey);
+    mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::accesskey, accessKey);
 
     if (accessKey.IsEmpty())
         return NS_OK;
@@ -987,7 +1023,7 @@ nsTextBoxFrame::RegUnregAccessKey(nsPresContext* aPresContext,
 
     // With a valid PresContext we can get the ESM 
     // and (un)register the access key
-    nsIEventStateManager *esm = aPresContext->EventStateManager();
+    nsIEventStateManager *esm = PresContext()->EventStateManager();
 
     PRUint32 key = accessKey.First();
     if (aDoReg)
