@@ -38,7 +38,7 @@
 #define BASE_H
 
 #ifdef DEBUG
-static const char BASE_CVS_ID[] = "@(#) $RCSfile: base.h,v $ $Revision: 1.17 $ $Date: 2005/01/20 02:25:45 $";
+static const char BASE_CVS_ID[] = "@(#) $RCSfile: base.h,v $ $Revision: 1.19 $ $Date: 2008/02/23 05:29:23 $";
 #endif /* DEBUG */
 
 /*
@@ -57,7 +57,6 @@ static const char BASE_CVS_ID[] = "@(#) $RCSfile: base.h,v $ $Revision: 1.17 $ $
 #endif /* NSSBASE_H */
 
 #include "plhash.h"
-#include "prthread.h"
 
 PR_BEGIN_EXTERN_C
 
@@ -520,6 +519,13 @@ extern const NSSError NSS_ERROR_INVALID_ARENA;
 #define nssArena_VERIFYPOINTER(p) (((NSSArena *)NULL == (p))?PR_FAILURE:PR_SUCCESS)
 #endif /* DEBUG */
 
+/*
+ * Private function to be called by NSS_Shutdown to cleanup nssArena 
+ * bookkeeping.
+ */
+extern PRStatus
+nssArena_Shutdown(void);
+
 /*
  * nssArenaHashAllocOps
  *
@@ -1407,42 +1413,6 @@ extern const NSSError NSS_ERROR_INVALID_POINTER;
 
 #define nsslibc_offsetof(str, memb) ((PRPtrdiff)(&(((str *)0)->memb)))
 
-/*
- * nss_NewThreadPrivateIndex
- * 
- */
-
-NSS_EXTERN PRStatus
-nss_NewThreadPrivateIndex
-(
-  PRUintn *ip,
-  PRThreadPrivateDTOR dtor
-);
-
-/*
- * nss_GetThreadPrivate
- *
- */
-
-NSS_EXTERN void *
-nss_GetThreadPrivate
-(
-  PRUintn i
-);
-
-/*
- * nss_SetThreadPrivate
- *
- */
-
-NSS_EXTERN void
-nss_SetThreadPrivate
-(
-  PRUintn i,
-  void *v
-);
-
-
 PR_END_EXTERN_C
 
 #endif /* BASE_H */
